--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

-- Purpose:
-- . Provide an array of complex adders with streaming I/O
-- Description:
-- . Adds snk_in_2arr_2[i][0] to snk_in_2arr_2[i][1] yielding 
--   src_out_arr[i] for i in 0..g_nof_inputs-1.

ENTITY corr_adder IS
  GENERIC (
    g_nof_inputs  : NATURAL
   ); 
  PORT (
    rst            : IN  STD_LOGIC;
    clk            : IN  STD_LOGIC;

    snk_in_2arr_2  : IN  t_dp_sosi_2arr_2(g_nof_inputs-1 DOWNTO 0);

    src_out_arr   : OUT t_dp_sosi_arr(g_nof_inputs-1 DOWNTO 0)
  );
END corr_adder;

ARCHITECTURE str OF corr_adder IS

BEGIN

  gen_common_complex_add_sub : FOR i IN 0 TO g_nof_inputs-1 GENERATE
    u_common_complex_add_sub : ENTITY common_lib.common_complex_add_sub
    GENERIC MAP (
      g_direction       => "ADD",
      g_representation  => "SIGNED",
      g_pipeline_input  => 0,
      g_pipeline_output => 1,
      g_in_dat_w        => 36,
      g_out_dat_w       => 64
    )
    PORT MAP (
      clk        => clk,
      in_ar      => snk_in_2arr_2(i)(0).re(35 DOWNTO 0),
      in_ai      => snk_in_2arr_2(i)(0).im(35 DOWNTO 0),
      in_br      => snk_in_2arr_2(i)(1).re(35 DOWNTO 0),
      in_bi      => snk_in_2arr_2(i)(1).im(35 DOWNTO 0),
      out_re     => src_out_arr(i).re(63 DOWNTO 0),
      out_im     => src_out_arr(i).im(63 DOWNTO 0)
    );     
  END GENERATE;

END str;
