--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, technology_lib, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.tech_transceiver_component_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;

ENTITY tech_transceiver_gx IS 
  GENERIC(
    g_technology            : NATURAL := c_tech_select_default;
    g_data_w                : NATURAL := 32; -- Supported: 32, 16.
    g_nof_gx                : NATURAL;
    g_mbps                  : NATURAL;  -- Supported: 6250, 5000, 3125, 2500 when g_data_w=32, else 2500 when g_data_w=16
    g_sim                   : BOOLEAN;
    g_tx                    : BOOLEAN;
    g_rx                    : BOOLEAN
  );      
  PORT(
    cal_rec_clk             : IN  STD_LOGIC;
    tr_clk                  : IN  STD_LOGIC;

    rx_clk                  : OUT STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);
    rx_rst                  : OUT STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);
    rx_sosi_arr             : OUT t_dp_sosi_arr(g_nof_gx-1 DOWNTO 0);
    rx_siso_arr             : IN  t_dp_siso_arr(g_nof_gx-1 DOWNTO 0);

    tx_clk                  : OUT STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);
    tx_rst                  : OUT STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);
    tx_sosi_arr             : IN  t_dp_sosi_arr(g_nof_gx-1 DOWNTO 0);
    tx_siso_arr             : OUT t_dp_siso_arr(g_nof_gx-1 DOWNTO 0);
   
    --Serial I/O
    rx_datain               : IN  STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);
    tx_dataout              : OUT STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);   

    -- Ctrl&stat
    tx_state                : OUT STD_LOGIC_VECTOR(2*g_nof_gx-1 DOWNTO 0);
    tx_align_en             : IN  STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0);

    rx_state                : OUT STD_LOGIC_VECTOR(2*g_nof_gx-1 DOWNTO 0);
    rx_align_en             : IN  STD_LOGIC_VECTOR(g_nof_gx-1 DOWNTO 0)
  );

END tech_transceiver_gx;

ARCHITECTURE str OF tech_transceiver_gx IS
BEGIN
 
  gen_ip_stratixiv : IF g_technology=c_tech_stratixiv GENERATE
    u0 : ENTITY work.tech_transceiver_gx_stratix_iv
    GENERIC MAP (g_data_w, g_nof_gx, g_mbps, g_sim, g_tx, g_rx)
    PORT MAP (cal_rec_clk, tr_clk, rx_clk, rx_rst, rx_sosi_arr, rx_siso_arr, tx_clk, tx_rst, tx_sosi_arr, tx_siso_arr, rx_datain, tx_dataout, tx_state, tx_align_en, rx_state, rx_align_en);
  END GENERATE;
  
END str;

