from myhdl import *
from random import randrange
from block_gen import block_gen

ACTIVE_LOW, INACTIVE_HIGH = 0, 1

def tb_block_gen():
    count, enable, clock = [Signal(intbv(0)) for i in range(3)]
    reset = ResetSignal(0, active=ACTIVE_LOW, async=True)

    bg = block_gen(count, enable, clock, reset, n=4)

    HALF_PERIOD = delay(10)

    @always(HALF_PERIOD)
    def clockGen():
        clock.next = not clock

    @instance
    def stimulus():
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for i in range(12):
            enable.next = min(1, randrange(3))
            yield clock.negedge
        raise StopSimulation

    @instance
    def monitor():
        print "enable  count"
        yield reset.posedge
        while 1:
            yield clock.posedge
            yield delay(1)
            print "   %s      %s" % (enable, count)

    return clockGen, stimulus, bg, monitor


# Optie 1: alleen print output
#tb = tb_block_gen()
#Simulation(tb).run()

# Optie 2: waveform
tb = traceSignals(tb_block_gen)
sim = Simulation(tb)
sim.run()
