import multiprocessing as mp
import time

class Port(object):
    def __init__(self):
        self.pipe_end = None

    #'>' operator: Connect
    def __gt__(self, other):
        return self.__connect__(other)

    def __connect__(self, other):
        snk_end, src_end = mp.Pipe(duplex=False)
        self.pipe_end  = src_end
        other.pipe_end = snk_end
        return

    def send(self, item):
       self.pipe_end.send(item) 

    def recv(self):
        while True:
            try:
                yield self.pipe_end.recv()
            except EOFError:
                break


#class Connection():
#    def __init__(self):
#        self.src_end, self.snk_end = mp.Pipe()

class Composite(object):
    def __init__(self, components=[], connections=[]):
        self.inst_name = None
        self.file_name = None
        self.components = components
        self.connections = connections

    def start(self):
        for component in self.components:
            print 'Starting', component
            component.start()

    def terminate(self):
        for component in self.components:
            print 'Terminating', component
            component.terminate()

    def run_time(self, time_sec):
        self.start()
        time.sleep(time_sec)
        self.terminate()

class Component(mp.Process):
    def __init__(self):
        mp.Process.__init__(self)
        self.inst_name = None
        self.file_name = None

class Counter(Component):
    def __init__(self):
        Component.__init__(self)
        self.src = Port()
    def run(self):
        for item in xrange(10):
            print 'sending', item
            self.src.send(item) 

class Through(Component):
    def __init__(self):
        Component.__init__(self)
        self.src = Port()
        self.snk = Port()
    def run(self):
        for item in self.snk.recv():
            print 'forwarding', item
            self.src.send(item) 

class Printer(Component):
    def __init__(self):
        Component.__init__(self)
        self.snk = Port() 
    def run(self):
        for item in self.snk.recv():
            print 'receiving', item

a = Counter()
b = Through()
c = Printer()

ab = a.src > b.snk
bc = b.src > c.snk

d = Composite([a,b,c], [ab,bc])

d.run_time(4)



